from alembic import command
import unittest
import json

from app import app, db, migrate
from ki.actions import seed
from ki.models import User


class TestProfileEndpoint(unittest.TestCase):
    maxDiff = None

    def setUp(self):
        app.debug = True
        app.config["TESTING"] = True
        app.config["SQLALCHEMY_DATABASE_URI"] = "sqlite:///:memory:"
        self.client = app.test_client()

        with app.app_context():
            config = migrate.get_config()
            command.upgrade(config, "head")

        seed(True)

    def tearDown(self):
        db.drop_all()
        db.engine.dispose()

    def test_update_profile(self):
        login_data = {"username": "peter", "password": "geheim"}
        login_response = self.client.post("/users/login",
                                          data=json.dumps(login_data),
                                          content_type="application/json")

        self.assertEqual(login_response.status_code, 200)
        self.assertIn("token", login_response.json)

        data = {
            "pronouns": "Monsieur",
            "volunteerwork": "ja",
            "freetext": "Hallo",
        }
        response = self.client.post("/users/1/profile",
                                    data=json.dumps(data),
                                    content_type="application/json",
                                    headers={
                                        "Authorization": "Bearer " +
                                        login_response.json["token"]
                                    })

        self.assertEqual(response.status_code, 200)
        with app.app_context():
            user = User.query.filter(User.id == 1).first()
            profile = user.profile
            self.assertEqual("Monsieur", profile.pronouns)
            self.assertEqual("ja", profile.volunteerwork)
            self.assertEqual("Hallo", profile.freetext)

    def test_get_profile(self):
        login_data = {"username": "peter", "password": "geheim"}
        login_response = self.client.post("/users/login",
                                          data=json.dumps(login_data),
                                          content_type="application/json")

        self.assertEqual(login_response.status_code, 200)
        self.assertIn("token", login_response.json)

        response = self.client.get("/users/1/profile",
                                   headers={
                                       "Authorization": "Bearer " +
                                       login_response.json["token"]
                                   })

        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(
            response.json, {
                "profile": {
                    "user_id": 1,
                    "nickname": "peternichtlustig",
                    "pronouns": "Herr Dr. Dr.",
                    "freetext": "Ich mag Kaffee",
                    "volunteerwork": "Gartenverein",
                    "address": {
                        "additional": "Hinterhaus",
                        "city": "Bielefeld",
                        "country": "Deutschland",
                        "house_number": "23i",
                        "id": 1,
                        "name": "Peter Nichtlustig",
                        "postcode": "13337",
                        "profile_id": 1,
                        "street": "Waldweg"
                    },
                    "contacts": [{
                        "id": 1,
                        "profile_id": 1,
                        "contacttype": {
                            "id": 1,
                            "name": "Matrix"
                        },
                        "content": "@peter:wtf-eg.de"
                    }],
                    "skills": [{
                        "profile_id": 1,
                        "skill": {
                            "id": 3,
                            "name": "Python",
                            "icon_url": "/skills/3/icon"
                        },
                        "level": 5
                    }],
                    "languages": [{
                        "profile_id": 1,
                        "language": {
                            "id": "de",
                            "name": "Deutsch",
                            "icon_url": "/languages/de/icon"
                        },
                        "level": 5
                    }, {
                        "profile_id": 1,
                        "language": {
                            "id": "fr",
                            "name": "Französisch",
                            "icon_url": "/languages/fr/icon"
                        },
                        "level": 3
                    }]
                }
            })


if __name__ == "main":
    unittest.main()
